#include "test.hpp"

int main()
{
    using namespace bast::opcodes;
    using namespace bast::types;

    using builder_t = bast::builder::program;
    using eval_t = bast::eval< bast::exec_tag, bast::exec_state, brq::unit_t >;

    builder_t b;
    brq::unit_t heap;

    auto test_simple = [=]( builder_t &b, auto expect ) mutable
    {
        auto p = b.build();
        auto e = eval_t( p, heap );
        auto s = bast::exec_state();

        e.block( s, { 0 }, []( bast::label w ) { ASSERT( false, w.v ); } );
        ASSERT( !s.empty() );
        ASSERT_EQ( s.top< decltype( expect ) >(), expect );
        b.clear();
    };

    brq::test_case( "word_add" ) = [=]() mutable
    {
        b.add_block() = { 13, 15, bv_add< word > };
        test_simple( b, 28 );
    };

    brq::test_case( "byte_add" ) = [=]() mutable
    {
        b.add_block() = { 13, bv_const< byte >,
                          15, bv_const< byte >, bv_add< byte > };
        test_simple( b, 28 );
    };

    brq::test_case( "byte_signed" ) = [=]() mutable
    {
        b.add_block() = { 13, byte_const, unsigned( -1 ), byte_const, byte_sdiv };
        test_simple( b, uint8_t( -13 ) );
        b.add_block() = { 13, byte_const, unsigned( -1 ), byte_const, byte_udiv };
        test_simple( b, 0 );
    };

    brq::test_case( "byte_g_tt" ) = [=]() mutable
    {
        b.add_block() = { 7, byte_const, bv_guard_nonzero< byte >, 1 };

        auto p = b.build();
        auto e = eval_t( p, heap );
        auto s = bast::exec_state();

        e.block( s, { 0 }, []( bast::label w ) { ASSERT( false, w.v ); } );
        ASSERT( !s.empty() );
        ASSERT_EQ( s.top< bast::word >(), 1 );
    };

    brq::test_case( "byte_g_ff" ) = [=]() mutable
    {
        b.add_block() = { 7, byte_const, bv_guard_zero< byte >, 1 };

        auto p = b.build();
        auto e = eval_t( p, heap );
        auto s = bast::exec_state();

        e.block( s, { 0 }, []( bast::label w ) { ASSERT( false, w.v ); } );
        ASSERT( s.empty() );
    };
}
